#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import logging

import requests
from coshed_model.naming import cfapp_base_url

ENVIRONMENT_ITEMS = ("prod", "dev", "qa")

DEFAULT_ENVIRONMENT = "dev"

DUMMY_API_KEY = "you are superior in only one respect -- you are better at dying"


class SPOFClient:
    def __init__(self, hostname, *args, **kwargs):
        self.log = logging.getLogger(__name__)
        self.api_key_prefix = kwargs.get("api_key_prefix", "SPOF_API_KEY")
        self.env_name = kwargs.get("env_name", "prod")
        api_keys = dict()
        self._session_store = dict()
        self.roster = dict()

        if kwargs.get("api_keys"):
            api_keys = kwargs.get("api_keys")
        else:
            for key in ENVIRONMENT_ITEMS:
                env_key = "{:s}_{!s}".format(self.api_key_prefix, key.upper())

                if self.api_key_prefix in os.environ:
                    env_key = self.api_key_prefix
                    self.log.debug(
                        "Using {env_key!s} for {key!s}".format(env_key=env_key, key=key)
                    )

                try:
                    api_keys[key] = os.environ[env_key]
                except KeyError:
                    self.log.warning(
                        "No API Key for {!s} found in environment variable {!s}".format(
                            key, env_key
                        )
                    )

        for env_name in ENVIRONMENT_ITEMS:
            self.roster[env_name] = dict()
            if not api_keys.get(env_name):
                api_keys[env_name] = DUMMY_API_KEY

            session_data = dict(
                base_url=cfapp_base_url(hostname, env_name=env_name),
                session=requests.Session(),
            )
            headers = dict(accept="application/json", authorization=api_keys[env_name])
            session_data["session"].headers.update(headers)
            # self.log.debug(session_data["session"].headers)
            self._session_store[env_name] = session_data



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import pendulum

from coshed_model.spof_client import SPOFClient
from coshed_model.spof_client import DEFAULT_ENVIRONMENT, ENVIRONMENT_ITEMS


class SPOFNotificationNotification(SPOFClient):
    def __init__(self, hostname, *args, **kwargs):
        super().__init__(hostname, *args, **kwargs)

    def post_machine_message(
        self, occurrence, serial_number, content, env_name=None, **kwargs
    ):
        if env_name is None:
            env_name = DEFAULT_ENVIRONMENT
        assert env_name in ENVIRONMENT_ITEMS
        sender = kwargs.get("sender", "ALARM_MONITORING_FEATURE")
        category = kwargs.get("category", "IndexAlarm")
        payload = {
            "notificationText": content,
            "source": sender,
            "messageCreatedTimestamp": pendulum.now().to_rfc3339_string(),
            "sourceEventTimestamp": occurrence.to_rfc3339_string(),
            "category": category,
        }
        s_data = self._session_store[env_name]
        the_url = "{base_url}/v1/{serial_number}/MONITORING_SERVICE/PostMachineMessage".format(
            base_url=s_data["base_url"], serial_number=serial_number
        )

        try:
            req = s_data["session"].post(the_url, json=payload)

            if req.status_code != 200:
                self.log.warning("Got HTTP Status Code {!r}".format(req.status_code))

            return req.status_code == 200
        except Exception as exc:
            self.log.error(exc)

        return False


if __name__ == "__main__":
    snn = SPOFNotificationNotification(
        "ix-api-manager", api_key_prefix="SPOF_SNN_API_KEY"
    )
    snn.post_machine_message(
        pendulum.now().subtract(minutes=17),
        "42424242",
        "<script>alert('gotcha');</script>",
        env_name="dev",
        category="SuperFieserAlarm",
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from coshed_model.spof_client import SPOFClient
from coshed_model.spof_client import DEFAULT_ENVIRONMENT, ENVIRONMENT_ITEMS


class SPOFRoster(SPOFClient):
    def __init__(self, hostname, *args, **kwargs):
        super().__init__(hostname, *args, **kwargs)
        self.arn_prefix = kwargs.get("arn_prefix", "OH:IF:ONLY:WE:KNEW:")

    def fetch(self, serial_number, env_name=None):
        data = list()
        if env_name is None:
            env_name = DEFAULT_ENVIRONMENT
        s_data = self._session_store[env_name]
        the_url = "{base_url}/v1/{serial_number}/NOTIFICATION_SERVICE/NotificationsForMachine".format(
            base_url=s_data["base_url"], serial_number=serial_number
        )

        try:
            req = s_data["session"].get(the_url)

            for raw in req.json():
                item = dict(
                    topic_arn=self.arn_prefix + raw["id"],
                    user_id=raw["userId"],
                )
                # self.log.debug(raw)
                data.append(item)
        except Exception:
            return None

        return data

    def __getitem__(self, key):
        if isinstance(key, tuple):
            env_name, item_key = key
        else:
            env_name = DEFAULT_ENVIRONMENT
            item_key = key

        if item_key not in self.roster[env_name]:
            value = self.fetch(item_key, env_name=env_name)
            if value is not None:
                self[key] = value

        return self.roster[env_name][item_key]

    def __setitem__(self, key, value):
        if isinstance(key, tuple):
            env_name, item_key = key
        else:
            env_name = DEFAULT_ENVIRONMENT
            item_key = key

        self.roster[env_name][item_key] = value

    def get_user_items(self, serial_number, env_name):
        assert env_name in ENVIRONMENT_ITEMS

        try:
            return self[(env_name, serial_number)]
        except KeyError:
            return []



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
.. seealso::

    * https://www.sqlitetutorial.net/sqlite-trigger/
    * https://www.sqlitetutorial.net/sqlite-replace-statement/
    * https://www.sqlitetutorial.net/sqlite-functions/sqlite-coalesce/
    * https://www.sqlite.org/inmemorydb.html
    * https://docs.sqlalchemy.org/en/13/dialects/sqlite.html#connect-strings
    * https://docs.sqlalchemy.org/en/14/core/compiler.html#utc-timestamp-function
    * https://stackoverflow.com/questions/13370317/sqlalchemy-default-datetime
    * https://sqlite.org/lang_datefunc.html

Tables
------

.. code-block:: sql
   :linenos:

    CREATE TABLE alarm_log (
        id INTEGER NOT NULL,
        alarm_id TEXT NOT NULL,
        threshold INTEGER DEFAULT 3,
        threshold_seconds INTEGER DEFAULT 0,
        shot INTEGER DEFAULT '0',
        occurrence DATETIME DEFAULT (CURRENT_TIMESTAMP),
        PRIMARY KEY (id)
    );

.. code-block:: sql
   :linenos:

    CREATE TABLE alarms (
        id text PRIMARY KEY,
        threshold INTEGER DEFAULT 3,
        count INTEGER DEFAULT 0,
        occurrence DATE DEFAULT CURRENT_TIMESTAMP
    );

.. code-block:: sql
   :linenos:

    CREATE TABLE trespass (
        id INTEGER PRIMARY KEY,
        alarm_id TEXT,
        threshold INTEGER,
        occurrence DATE,
        discovery DATE DEFAULT CURRENT_TIMESTAMP
    );

.. code-block:: sql
   :linenos:

    CREATE TABLE trespass_database_information (
        id INTEGER NOT NULL, 
        key TEXT NOT NULL, 
        value TEXT NOT NULL, 
        dt DATETIME DEFAULT (CURRENT_TIMESTAMP), 
        PRIMARY KEY (id)
    )

Triggers
--------

.. code-block:: sql
   :linenos:

    CREATE TRIGGER trespass_insert
    AFTER INSERT ON alarms
    WHEN
        new.count >= new.threshold
    BEGIN
        INSERT INTO trespass (alarm_id, threshold, occurrence) VALUES(new.id, new.threshold, new.occurrence);
        UPDATE alarms SET count=0 WHERE id=new.id;
        UPDATE alarm_log SET shot=1 WHERE alarm_id=new.id;
    END;

.. code-block:: sql
   :linenos:

    CREATE TRIGGER drop_outdated
    AFTER INSERT ON alarm_log
    BEGIN
        DELETE FROM alarm_log WHERE alarm_id = new.alarm_id AND id != new.id AND new.threshold_seconds > 0 AND strftime("%s", occurrence) + new.threshold_seconds - strftime("%s", new.occurrence) < 0;
        DELETE FROM alarm_log WHERE alarm_id = new.alarm_id AND id != new.id AND occurrence=new.occurrence;

        INSERT OR REPLACE INTO alarms (id, count, threshold, occurrence) VALUES(new.alarm_id, (SELECT COUNT(*) FROM alarm_log WHERE alarm_id=new.alarm_id AND shot=0), new.threshold, new.occurrence);
    END;

Example insert
--------------

.. code-block:: sql
   :linenos:

    INSERT
    INTO alarm_log (alarm_id, occurrence, threshold, threshold_seconds) 
    VALUES("1", "2021-01-01T00:00:00Z", 3, 3600);

"""
import datetime

import pendulum
from sqlalchemy import event, DDL
from sqlalchemy import Column, Integer, Text, DateTime
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.sql import expression
from sqlalchemy.ext.compiler import compiles
from sqlalchemy import types
from sqlalchemy.sql import text

Base = declarative_base()

# https://stackoverflow.com/questions/13370317/sqlalchemy-default-datetime
# https://docs.sqlalchemy.org/en/14/core/compiler.html#utc-timestamp-function
class utcnow(expression.FunctionElement):
    type = DateTime()


# https://docs.sqlalchemy.org/en/14/core/compiler.html#utc-timestamp-function
@compiles(utcnow, "sqlite")
def sqlite_utcnow(element, compiler, **kw):
    return "CURRENT_TIMESTAMP"


class DateTimeUTC(types.TypeDecorator):
    impl = types.DateTime
    LOCAL_TIMEZONE = datetime.datetime.utcnow().astimezone().tzinfo

    def process_bind_param(self, value: datetime.datetime, dialect):
        if value.tzinfo is None:
            value = value.astimezone(self.LOCAL_TIMEZONE)

        return value.astimezone(pendulum.UTC)

    def process_result_value(self, value, dialect):
        if value.tzinfo is None:
            return value.replace(tzinfo=pendulum.UTC)

        return value.astimezone(pendulum.UTC)


#: default value - threshold number
DEFAULT_THRESHOLD = 3

#: default value - timeframe threshold
DEFAULT_THRESHOLD_SECONDS = 0

#: database schema version indicator
DATABASE_SCHEMA_VERSION = 2


class AlarmLog(Base):
    """
    Alarm events.

    Attributes:
        id (str): Alarm Identifier
        threshold (int): Threshold number for counting as trespassing
        threshold_seconds (int): timeframe threshold for counting as trespassing
        occurrence(datetime.datetime): occurrence datetime
        shot(int): Indicator if entry has already been considered

    """

    __tablename__ = "alarm_log"

    id = Column(Integer, primary_key=True, autoincrement=True)
    alarm_id = Column(Text, nullable=False)
    threshold = Column(Integer, server_default=str(DEFAULT_THRESHOLD))
    threshold_seconds = Column(
        Integer, server_default=str(DEFAULT_THRESHOLD_SECONDS)
    )
    shot = Column(Integer, server_default="0")
    occurrence = Column(DateTimeUTC, server_default=utcnow())

    def __init__(
        self, alarm_id, occurrence=None, threshold=None, threshold_seconds=None
    ):
        self.alarm_id = alarm_id

        if threshold is not None:
            self.threshold = threshold

        if threshold_seconds is not None:
            self.threshold_seconds = threshold_seconds

        if occurrence is not None:
            self.occurrence = occurrence

    def __str__(self) -> str:
        return "<{klass} {alarm_id!r} {occurrence!s}>".format(
            klass=self.__class__.__name__,
            alarm_id=self.alarm_id,
            occurrence=self.occurrence,
        )


class Alarm(Base):
    """
    Alarm counter.

    Attributes:
        id (str): Alarm Identifier
        threshold (int): Threshold number for counting as trespassing
        count (int): occurrences count
        occurrence(datetime.datetime): most recent occurrence datetime

    """

    __tablename__ = "alarms"

    id = Column(Text, primary_key=True)
    threshold = Column(Integer, server_default=str(DEFAULT_THRESHOLD))
    count = Column(Integer, server_default="0")
    occurrence = Column(DateTimeUTC, server_default=utcnow())

    def __init__(self, id, threshold=None, count=None, occurrence=None):
        self.id = id

        if threshold is not None:
            self.threshold = threshold

        if not count:
            count = 1
        self.count = count

        if occurrence is not None:
            self.occurrence = occurrence


class Trespass(Base):
    """
    Trespassing events.

    Attributes:
        id (str): Row ID
        threshold (int): Threshold number for counting as trespassing
        threshold_seconds (int): timeframe threshold for counting as trespassing
        occurrence (datetime.datetime): most recent occurrence datetime
        discovery (datetime.datetime): datetime of trespassing discovery

    """

    __tablename__ = "trespass"

    id = Column(Integer, primary_key=True)
    alarm_id = Column(Text, nullable=False)
    threshold = Column(Integer)
    threshold_seconds = Column(Integer)
    occurrence = Column(DateTimeUTC)
    discovery = Column(DateTimeUTC, server_default=utcnow())

    def __init__(self, alarm_id):
        self.alarm_id = alarm_id

    def __str__(self) -> str:
        return "<{klass} {alarm_id!r} {occurrence!s}>".format(
            klass=self.__class__.__name__,
            alarm_id=self.alarm_id,
            occurrence=self.occurrence,
        )

    @property
    def json(self):
        dt_occurrence = pendulum.instance(self.occurrence).in_tz(pendulum.UTC)
        dt_discovery = pendulum.instance(self.discovery).in_tz(pendulum.UTC)

        return dict(
            occurrence=dt_occurrence.to_rfc3339_string(),
            discovery=dt_discovery.to_rfc3339_string(),
            alarm_id=self.alarm_id,
        )


class TresspassDatabaseInformation(Base):
    """
    (Internal) Database key/value pairs for storing meta data.

    Attributes:
        id (str): Row ID
        key (str): key
        value (str): value

    """

    __tablename__ = "trespass_database_information"

    id = Column(Integer, primary_key=True)
    key = Column(Text, nullable=False)
    value = Column(Text, nullable=False)
    dt = Column(DateTimeUTC, server_default=utcnow())

    def __init__(self, key, value):
        self.key = key
        self.value = value


event.listen(
    Trespass.__table__,
    "after_create",
    DDL(
        """CREATE TRIGGER trespass_insert
        AFTER INSERT ON alarms
        WHEN 
                new.count >= new.threshold
        BEGIN
            INSERT INTO trespass (alarm_id, threshold, occurrence) VALUES(new.id, new.threshold, new.occurrence);
            UPDATE alarms SET count=0 WHERE id=new.id;
            UPDATE alarm_log SET shot=1 WHERE alarm_id=new.id;
        END;
        """
    ),
)

event.listen(
    AlarmLog.__table__,
    "after_create",
    DDL(
        """CREATE TRIGGER drop_outdated
        AFTER INSERT ON alarm_log
        BEGIN
            DELETE FROM alarm_log WHERE alarm_id = new.alarm_id AND id != new.id AND new.threshold_seconds > 0 AND strftime("%%s", occurrence) + new.threshold_seconds - strftime("%%s", new.occurrence) < 0;
            DELETE FROM alarm_log WHERE alarm_id = new.alarm_id AND id != new.id AND occurrence=new.occurrence;

            INSERT OR REPLACE INTO alarms (id, count, threshold, occurrence) VALUES(new.alarm_id, (SELECT COUNT(*) FROM alarm_log WHERE alarm_id=new.alarm_id AND shot=0), new.threshold, new.occurrence);
        END;
        """
    ),
)

event.listen(
    TresspassDatabaseInformation.__table__,
    "after_create",
    DDL(
        "INSERT INTO trespass_database_information (key, value) VALUES('schema_version', '{!s}');".format(
            DATABASE_SCHEMA_VERSION
        )
    ),
)

event.listen(
    TresspassDatabaseInformation.__table__,
    "after_create",
    DDL(
        "INSERT INTO trespass_database_information (key, value) VALUES('created', '{!s}');".format(
            pendulum.now(pendulum.UTC).to_datetime_string()
        )
    ),
)


def add_alarm_occurrence(session, alarm_id, occurrence=None):
    stmt_args = dict(id=alarm_id)

    if occurrence:
        statement = text(
            """
            INSERT INTO alarm_log (alarm_id, occurrence)
            VALUES (
                :id,
                :occurrence
            )
        """
        )
        x = pendulum.instance(occurrence)
        stmt_args["occurrence"] = x.in_tz(pendulum.UTC).to_datetime_string()
    else:
        statement = text(
            """
            INSERT INTO alarm_log (alarm_id)
            VALUES (
                :id
            )
        """
        )
    session.execute(statement, stmt_args)
    session.commit()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import logging

import pendulum
from djali.dynamodb import DynamoController
from coshed_model.naming import environment_specific_name


TID = "alarm_latest_occurrence"
DEFAULT_ENV_NAME = "prod"
DEFAULT_REGION = "eu-central-1"
DEFAULT_TABLE_DATA = "alarmed"
DEFAULT_TABLE_RULESET = "alarmed_id"

DATASET_VERSION = 3


class UserDataControl:
    def __init__(self, user_id, namespaces=None, *args, **kwargs):
        if namespaces is None:
            namespaces = ()
        self.log = logging.getLogger(__name__)
        table_data = kwargs.get("table_data", DEFAULT_TABLE_DATA)
        table_ruleset = kwargs.get("table_ruleset", DEFAULT_TABLE_RULESET)
        region_name = kwargs.get("region_name", DEFAULT_REGION)
        self.env_name = kwargs.get("env_name", DEFAULT_ENV_NAME)
        self.user_id = str(user_id)
        self.namespaces = namespaces
        self.dc_data = DynamoController(table_data, region_name=region_name)
        self.dc_ruleset = DynamoController(table_ruleset, region_name=region_name)
        self.user_data = dict()
        self.user_rulesets = dict()

    @property
    def data_key(self):
        return environment_specific_name(
            "user-settings-{user_id}".format(user_id=self.user_id),
            env_name=self.env_name,
        )

    @property
    def data(self):
        if "dt" in self.user_data:
            self._sanitise_user_data_structure()
            return self.user_data

        self.load()

        return self.user_data

    def _sanitise_user_data_structure(self):
        try:
            self.user_data[TID]
        except KeyError:
            self.user_data[TID] = dict()

        for namespace in self.namespaces:
            try:
                self.user_data[TID][namespace]
            except KeyError:
                self.user_data[TID][namespace] = dict()

        self.user_data["version"] = DATASET_VERSION
        self.user_data["user_id"] = self.user_id

    def generate_ruleset_key(self, namespace):
        derived = environment_specific_name(self.user_id, env_name=self.env_name)
        return ".".join((namespace, derived))

    def __str__(self):
        portions = ["<{klass}".format(klass=self.__class__.__name__), self.data_key]
        for key in sorted(self.namespaces):
            portions.append(
                "{key}={val}".format(key=key, val=self.generate_ruleset_key(key))
            )
        return " ".join(portions) + ">"

    def _fetch_ruleset(self, namespace):
        key = self.generate_ruleset_key(namespace)
        self.log.info(
            "Fetching ruleset for {namespace}: {key!r}".format(
                namespace=namespace, key=key
            )
        )
        data = self.dc_ruleset[key]

        return data

    def load(self):
        default_data = dict(
            dt=pendulum.now().to_rfc3339_string(), version=3, user_id=self.user_id
        )

        try:
            data = self.dc_data[self.data_key]
            self.log.info(
                "Loaded data using {data_key!r}".format(data_key=self.data_key)
            )
        except KeyError:
            data = default_data
            self.log.warning(
                "No data available using {data_key!r}. Using default!".format(
                    data_key=self.data_key
                )
            )

        self.user_data = data
        self._sanitise_user_data_structure()

    def save(self):
        self.log.info("Saving data using {data_key!r}".format(data_key=self.data_key))
        self.user_data["dt"] = pendulum.now().to_rfc3339_string()
        self.dc_data[self.data_key] = self.user_data

    def drop(self):
        self.log.info("Dropping data using {data_key!r}".format(data_key=self.data_key))

        try:
            del self.dc_data[self.data_key]
        except KeyError:
            pass

    def __getitem__(self, key):
        if key in self.namespaces:
            try:
                return self.user_rulesets[key]
            except KeyError:
                try:
                    self.user_rulesets[key] = self._fetch_ruleset(key)
                except KeyError:
                    self.user_rulesets[key] = dict()

                return self.user_rulesets[key]

    def get_latest_occurrence(self, alarm_id, namespace, serial_number):
        rv = None

        self.log.info(
            "Retrieve latest occurrence for {alarm_id!r} on [{namespace}] {serial_number}".format(
                alarm_id=alarm_id, namespace=namespace, serial_number=serial_number
            )
        )

        try:
            rv = pendulum.parse(self.data[TID][namespace][alarm_id][serial_number])
        except KeyError:
            pass
        except Exception as exc:
            self.log.warning(exc)

        self.log.info("-> {!r}".format(rv))

        return rv

    def set_latest_occurrence(self, alarm_id, occurrence, namespace, serial_number):
        v = occurrence.to_rfc3339_string()

        self.log.info(
            "Set latest occurrence for {alarm_id!r} on [{namespace}] {serial_number}: {value}".format(
                alarm_id=alarm_id,
                namespace=namespace,
                serial_number=serial_number,
                value=v,
            )
        )

        try:
            self.data[TID][namespace]
        except KeyError:
            self._sanitise_user_data_structure()
        except Exception as exc:
            self.log.error(
                "Oh, FUCK. Cannot set up data dictionary. This won't end well."
            )
            self.log.error(exc)

        try:
            self.data[TID][namespace][alarm_id]
        except KeyError:
            self.data[TID][namespace][alarm_id] = dict()

        self.data[TID][namespace][alarm_id][serial_number] = v



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import logging
import re

PATTERN_INVALID_CHAR = r"[^a-z0-9]"
REGEX_INVALID_CHAR = re.compile(PATTERN_INVALID_CHAR)

PATTERN_BLACKLISTED = r"^([nN][\-\\\/][aA])$"
REGEX_BLACKLISTED = re.compile(PATTERN_BLACKLISTED)

PATTERN_MAC_STRIPPED = r"^[a-f0-9]{12}$"
REGEX_MAC_STRIPPED = re.compile(PATTERN_MAC_STRIPPED)

PATTERN_VALID_HEX = r"^[a-f0-9]{4,}$"
REGEX_VALID_HEX = re.compile(PATTERN_VALID_HEX)


def valid_mac_addr_or_bust(mac_addr):
    """
    Make sure that provided value is a valid MAC address and return its
    stripped value, portions and vendor prefix

    Args:
        mac_addr (str): MAC address

    Returns:
        tuple: stripped value, portions, vendor prefix

    >>> valid_mac_addr_or_bust("00:30:de:ad:be:ef")
    ('0030deadbeef', [0, 48, 222, 173, 190, 239], '00:30:de')
    """
    mac_addr = mac_addr.lower()
    mac_portions = [int(x, 16) for x in mac_addr.split(":")]

    mac_stripped = "".join(["{:02x}".format(x) for x in mac_portions])
    vendor_prefix = ":".join(["{:02x}".format(x) for x in mac_portions[:3]])

    assert REGEX_MAC_STRIPPED.match(mac_stripped) is not None

    return mac_stripped, mac_portions, vendor_prefix


class BlacklistedError(ValueError):
    pass


def mangled_dns_value(value):
    """
    Mangle a serial number value in such ways that it can be used
    as a hostname in DNS.

    Args:
        value (str): input value

    Returns:
        str: mangled value

    >>> mangled_dns_value('')
    Traceback (most recent call last):
        ...
    ValueError: '' is an invalid hostname
    >>> mangled_dns_value("B+W 161689.00")
    'b-w-161689-00'
    >>> mangled_dns_value("C40-16671")
    'c40-16671'
    >>> mangled_dns_value("DG TEST")
    'dg-test'
    >>> mangled_dns_value("DMU50-1")
    'dmu50-1'
    >>> mangled_dns_value("Heller 58112")
    'heller-58112'
    >>> mangled_dns_value("Proj444001")
    'proj444001'
    >>> mangled_dns_value("SIM MBL65-1")
    'sim-mbl65-1'
    >>> mangled_dns_value("Simulator 1000011")
    'simulator-1000011'
    >>> mangled_dns_value("Simulator ST4")
    'simulator-st4'
    >>> mangled_dns_value("Waldrich 15091")
    'waldrich-15091'
    >>> mangled_dns_value("n/a")
    Traceback (most recent call last):
        ...
    BlacklistedError: 'n/a' is blacklisted
    >>> mangled_dns_value("00:0d:b9:3c:4a:70")
    '00-0d-b9-3c-4a-70'
    """
    mangled = str(value).lower().strip()

    if not value:
        raise ValueError("{!r} is an invalid hostname".format(value))

    if REGEX_BLACKLISTED.match(mangled):
        raise BlacklistedError("{!r} is blacklisted".format(value))

    mangled = re.sub(REGEX_INVALID_CHAR, " ", mangled)
    mangled = re.sub("\s+", "-", mangled.strip())

    return mangled


if __name__ == "__main__":
    import doctest

    logging.basicConfig(
        level=logging.INFO,
        format="%(asctime)s %(levelname)-8s %(message)s",
        datefmt="%Y-%m-%d %H:%M:%S",
    )
    # logging.basicConfig(loglevel=logging.DEBUG)
    (FAILED, SUCCEEDED) = doctest.testmod()
    print("[doctest] SUCCEEDED/FAILED: {:d}/{:d}".format(SUCCEEDED, FAILED))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
def cfapp_base_url(value, env_name=None, **kwargs):
    """
    
    >>> cfapp_base_url("crap")
    'https://crap.cfapps.eu10.hana.ondemand.com'
    >>> cfapp_base_url("crap", env_name="prod")
    'https://crap.cfapps.eu10.hana.ondemand.com'
    >>> cfapp_base_url("crap", env_name="dev")
    'https://crap-dev.cfapps.eu10.hana.ondemand.com'
    >>> cfapp_base_url("crap", env_name="dev", prefixed=True)
    'https://dev-crap.cfapps.eu10.hana.ondemand.com'
    """
    return "https://{hostname}.cfapps.eu10.hana.ondemand.com".format(
                hostname=environment_specific_name(
                    value, env_name=env_name, **kwargs
                )
            )

def environment_specific_name(value, env_name=None, **kwargs):
    """
    Generate an environment specific name for ``value``.

    Schema for determining the environment specific name is

    **Append environment name separated by a dash if environment name is not** ``prod``

    .. note::

        This function is case agnostic, returned values are lower case.

    Args:
        value: value
        env_name (str, optional): environment name

    Keyword Args:
        prefixed (bool): Force prefix mode (even if it is ``prod``)

    Returns:
        str: environment specific name

    >>> environment_specific_name(0)
    Traceback (most recent call last):
        ...
    AttributeError: 'int' object has no attribute 'lower'
    >>> environment_specific_name(None)
    Traceback (most recent call last):
        ...
    AttributeError: 'NoneType' object has no attribute 'lower'
    >>> environment_specific_name("x")
    'x'
    >>> environment_specific_name("X")
    'x'
    >>> environment_specific_name("x", env_name="prod")
    'x'
    >>> environment_specific_name("x", env_name="PROD")
    'x'
    >>> environment_specific_name("x", env_name="prod", prefixed=True)
    'prod-x'
    >>> environment_specific_name("x", env_name="PROD", prefixed=True)
    'prod-x'
    >>> environment_specific_name("x", env_name="qa")
    'x-qa'
    >>> environment_specific_name("x", env_name="dev")
    'x-dev'
    """
    if env_name is None:
        return value.lower()

    if kwargs.get("prefixed"):
        return "-".join((env_name, value)).lower()

    if env_name.lower() != "prod":
        return "-".join((value, env_name)).lower()

    return value.lower()


if __name__ == "__main__":
    import doctest

    (FAILED, SUCCEEDED) = doctest.testmod()
    print("[doctest] SUCCEEDED/FAILED: {:d}/{:d}".format(SUCCEEDED, FAILED))
